<?php
/**
 * @category    Pinpoint
 * @package     Pinpoint_Faq
 * @copyright   Copyright (c) 2019 Pinpoint Designs. (http://www.Pinpoint.co.uk)
 * @author      Ahmed Goda <hello@Pinpoint.co.uk>
 * @author      Emad ElShafie <emad@pinpointdesigns.co.uk>
 */

namespace Pinpoint\Faq\Block\Adminhtml\Faq\Edit;

use Magento\Backend\Block\Widget\Context;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Pinpoint\Faq\Api\Data\FaqInterface;
use Pinpoint\Faq\Api\FaqRepositoryInterface;
use Magento\Framework\View\Element\Template;
use Magento\Store\Api\WebsiteRepositoryInterface;

/**
 * Class Product
 */
class Product extends Template
{
    /**
     * @var Context
     */
    protected $context;

    /**
     * @var FaqRepositoryInterface
     */
    protected $faqRepository;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * Catalog product model
     *
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var WebsiteRepositoryInterface
     */
    protected $website;

    /**
     * @param Context $context
     * @param FaqRepositoryInterface $faqRepository
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     * @param WebsiteRepositoryInterface $website
     * @param array $data
     */
    public function __construct(
        Context $context,
        FaqRepositoryInterface $faqRepository,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        WebsiteRepositoryInterface $website,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->context            = $context;
        $this->faqRepository      = $faqRepository;
        $this->storeManager       = $storeManager;
        $this->productRepository  = $productRepository;
        $this->website = $website;
    }

    /**
     * Return the id of the selected item
     *
     * @return int|null
     */
    public function getItemId(): ?int
    {
        try {
            $faq = $this->faqRepository->getById(
                $this->context->getRequest()->getParam(FaqInterface::FAQ_ID, 0)
            );
            if ($faq instanceof FaqInterface) {
                return $faq->getId();
            }
        } catch (NoSuchEntityException $e) {
        }
        return null;
    }

    /**
     * Retrieve current product model
     *
     * @return ProductInterface
     * @throws NoSuchEntityException
     */
    public function getProduct()
    {
        $faq = $this->faqRepository->getById($this->getItemId());
        return $this->productRepository->getById($faq->getProductId(), false, $this->storeManager->getStore()->getId());
    }

    /**
     * @return string
     */
    public function getWebsiteStoreLabel()
    {
        try {
            $faq = $this->faqRepository->getById($this->getItemId());
            $storeId = $faq->getStoreId();
            if ($storeId === '0') {
                return __('All Websites');
            } else {
                    $storeInfo = $this->storeManager->getStore($storeId);
                    $storeLabel = $storeInfo->getName();
                    $websiteId = $storeInfo->getWebsiteId();
                    $websiteLabel = $this->website->getById($websiteId)->getName();
                    return __($websiteLabel).' - '.__($storeLabel);
            }
        } catch (\Exception $e) {
            return __('FAQ / Website not found');
        }
    }
}
