<?php
/**
 * @category    Pinpoint
 * @package     Pinpoint_Faq
 * @copyright   Copyright (c) 2019 Pinpoint Designs. (http://www.Pinpoint.co.uk)
 * @author      Ahmed Goda <hello@Pinpoint.co.uk>
 */

namespace Pinpoint\Faq\Controller\Adminhtml\Faq;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\Result\Json as JsonResult;
use Magento\Framework\Controller\Result\JsonFactory;
use Pinpoint\Faq\Api\FaqRepositoryInterface;
use Pinpoint\Faq\Api\Data\FaqInterface;
use Pinpoint\Faq\Model\ResourceModel\FaqFactory as FaqResourceFactory;

/**
 * Class InlineEdit
 */
class InlineEdit extends Action
{
    /**
     * Authorization level of a basic admin session
     */
    const ADMIN_RESOURCE = 'Pinpoint_Faq::faq';

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $jsonFactory;

    /**
     * @var FaqRepositoryInterface
     */
    protected $faqRepository;

    /**
     * @var FaqResourceFactory
     */
    private $faqResourceFactory;

    /**
     * @param Context            $context
     * @param FaqRepository      $faqRepository
     * @param FaqResourceFactory $faqResourceFactory
     * @param JsonFactory        $jsonFactory
     */
    public function __construct(
        Context $context,
        JsonFactory $jsonFactory,
        FaqRepositoryInterface $faqRepository,
        FaqResourceFactory $faqResourceFactory
    ) {
        parent::__construct($context);
        $this->jsonFactory = $jsonFactory;
        $this->faqRepository = $faqRepository;
        $this->faqResourceFactory = $faqResourceFactory;
    }

    /**
     * @return JsonResult
     */
    public function execute()
    {
        /** @var JsonResult $jsonResult */
        $error = false;
        $messages = [];
        $jsonResult = $this->jsonFactory->create();

        if ($this->getRequest()->getParam('isAjax')) {
            $items = $this->getRequest()->getParam('items', []);
            if (!count($items)) {
                $messages[] = __('Please correct the data sent.');
                $error = true;
            } else {
                foreach (array_keys($items) as $id) {
                    /** @var \Pinpoint\Faq\Model\Faq $faq */
                    $faq = $this->faqRepository->getById($id);
                    try {
                        $faq->setData(array_merge($faq->getData(), $items[$id]));
                        $this->faqResourceFactory->create()->save($faq);
                    } catch (\Exception $e) {
                        $messages[] = $this->getErrorWithFaqId(
                            $faq,
                            __($e->getMessage())
                        );
                        $error = true;
                    }
                }
            }
        }

        return $jsonResult->setData(
            [
                'messages' => $messages,
                'error' => $error
            ]
        );
    }

    /**
     * Add faq title to error message
     *
     * @param  FaqInterface $faq
     * @param  string       $errorText
     * @return string
     */
    protected function getErrorWithFaqId(FaqInterface $faq, $errorText): string
    {
        return '[FAQ ID: ' . $faq->getId() . '] ' . $errorText;
    }
}
